@@ -71,8 +71,12 @@ namespace MonoDevelop.Core.AddIns.Setup
 			get { return addInDirs; }
 		}
 		
-		string BinPath {
-			get { return Path.GetDirectoryName (Assembly.GetEntryAssembly().Location); } 
+		string BinPath
+		{
+			get
+			{
+				return PropertyService.EntryAssemblyDirectory;
+			} 
 		}
 		
 		string UserConfigPath {
@@ -119,7 +123,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 			PackageCollection toUninstall;
 			PackageDependencyCollection unresolved;
 			if (!Runtime.SetupService.ResolveDependencies (monitor, packs, out toUninstall, out unresolved)) {
-				monitor.ReportError ("Not all dependencies could be resolved.", null);
+				monitor.ReportError (GettextCatalog.GetString ("Not all dependencies could be resolved."), null);
 				return false;
 			}
 			
@@ -127,7 +131,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 			ArrayList uninstallPrepared = new ArrayList ();
 			bool rollback = false;
 			
-			monitor.BeginTask (GettextCatalog.GetString ("Installing addins..."), 100);
+			monitor.BeginTask (GettextCatalog.GetString ("Installing add-ins..."), 100);
 			
 			// Prepare install
 			
@@ -451,14 +455,14 @@ namespace MonoDevelop.Core.AddIns.Setup
 					if (ap.Addin.Id == otherap.Addin.Id) {
 						if (ap.IsUpgradeOf (otherap)) {
 							if (requested.Contains (otherap)) {
-								monitor.ReportError ("Can't install two versions of the same add-in: '" + ap.Addin.Name + "'.", null);
+								monitor.ReportError (GettextCatalog.GetString ("Can't install two versions of the same add-in: '") + ap.Addin.Name + "'.", null);
 								error = true;
 							} else {
 								packages.RemoveAt (k);
 							}
 						} else if (otherap.IsUpgradeOf (ap)) {
 							if (requested.Contains (ap)) {
-								monitor.ReportError ("Can't install two versions of the same add-in: '" + ap.Addin.Name + "'.", null);
+								monitor.ReportError (GettextCatalog.GetString ("Can't install two versions of the same add-in: '") + ap.Addin.Name + "'.", null);
 								error = true;
 							} else {
 								packages.RemoveAt (n);
@@ -466,7 +470,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 							}
 						} else {
 							error = true;
-							monitor.ReportError ("Can't install two versions of the same add-in: '" + ap.Addin.Name + "'.", null);
+							monitor.ReportError (GettextCatalog.GetString ("Can't install two versions of the same add-in: '") + ap.Addin.Name + "'.", null);
 						}
 						break;
 					}
@@ -565,6 +569,12 @@ namespace MonoDevelop.Core.AddIns.Setup
 			}
 		}
 		
+		public bool CheckInstalledAddin (string id, string version)
+		{
+			AddinSetupInfo addin = GetInstalledAddin (id);
+			return addin.Addin.SupportsVersion (version);
+		}
+		
 		public AddinSetupInfo[] GetInstalledAddins ()
 		{
 			ArrayList list = InternalGetInstalledAddins ();
@@ -687,7 +697,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 				SaveConfiguration ();
 				return rr;
 			} catch (Exception ex) {
-				monitor.ReportError ("The repository could not be registered", ex);
+				monitor.ReportError (GettextCatalog.GetString ("The repository could not be registered"), ex);
 				if (IsRepositoryRegistered (url))
 					UnregisterRepository (url);
 				return null;
@@ -814,17 +824,20 @@ namespace MonoDevelop.Core.AddIns.Setup
 		public void BuildRepository (IProgressMonitor monitor, string path)
 		{
 			string mainPath = Path.Combine (path, "main.mrep");
+			ArrayList allAddins = new ArrayList ();
+			
 			Repository rootrep = (Repository) ReadObject (mainPath, typeof(Repository));
 			if (rootrep == null) {
 				rootrep = new Repository ();
 			}
 			
-			BuildRepository (monitor, rootrep, path, "root.mrep");
+			BuildRepository (monitor, rootrep, path, "root.mrep", allAddins);
 			WriteObject (mainPath, rootrep);
+			GenerateIndexPage (rootrep, allAddins, path);
 			monitor.Log.WriteLine ("Updated main.mrep");
 		}
 		
-		void BuildRepository (IProgressMonitor monitor, Repository rootrep, string rootPath, string relFilePath)
+		void BuildRepository (IProgressMonitor monitor, Repository rootrep, string rootPath, string relFilePath, ArrayList allAddins)
 		{
 			DateTime lastModified = DateTime.MinValue;
 			
@@ -852,6 +865,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 					modified = true;
 					monitor.Log.WriteLine ("Added addin: " + fname);
 				}
+				allAddins.Add (entry);
 				
 				DateTime date = File.GetLastWriteTime (file);
 				if (date > lastModified)
@@ -884,10 +898,33 @@ namespace MonoDevelop.Core.AddIns.Setup
 			
 			foreach (string dir in Directory.GetDirectories (mainPath)) {
 				string based = dir.Substring (rootPath.Length + 1);
-				BuildRepository (monitor, rootrep, rootPath, Path.Combine (based, "main.mrep"));
+				BuildRepository (monitor, rootrep, rootPath, Path.Combine (based, "main.mrep"), allAddins);
 			}
 		}
 		
+		void GenerateIndexPage (Repository rep, ArrayList addins, string basePath)
+		{
+			StreamWriter sw = new StreamWriter (Path.Combine (basePath, "index.html"));
+			sw.WriteLine ("<html><body><head>");
+			sw.WriteLine ("<html><body>");
+			sw.WriteLine ("<link type='text/css' rel='stylesheet' href='md.css' />");
+			sw.WriteLine ("</head>");
+			sw.WriteLine ("<h1>MonoDevelop Add-in Repository</h1>");
+			if (rep.Name != null && rep.Name != "")
+				sw.WriteLine ("<h2>" + rep.Name + "</h2>");
+			sw.WriteLine ("<p>This is a list of add-ins available in this repository. ");
+			sw.WriteLine ("If you need information about how to install add-ins, please read <a href='http://www.monodevelop.com/Installing_Add-ins'>this</a>.</p>");
+			sw.WriteLine ("<table border=1><thead><tr><th>Add-in</th><th>Version</th><th>Description</th></tr></thead>");
+			
+			foreach (AddinRepositoryEntry entry in addins) {
+				sw.WriteLine ("<tr><td>" + entry.Addin.Id + "</td><td>" + entry.Addin.Version + "</td><td>" + entry.Addin.Description + "</td></tr>");
+			}
+			
+			sw.WriteLine ("</table>");
+			sw.WriteLine ("</body></html>");
+			sw.Close ();
+		}
+		
 		public void BuildPackage (IProgressMonitor monitor, string targetDirectory, params string[] filePaths)
 		{
 			foreach (string file in filePaths)
@@ -999,6 +1036,8 @@ namespace MonoDevelop.Core.AddIns.Setup
 			try {
 				XmlSerializer ser = new XmlSerializer (type);
 				return ser.Deserialize (r);
+			} catch {
+				return null;
 			} finally {
 				r.Close ();
 			}
@@ -1032,13 +1071,13 @@ namespace MonoDevelop.Core.AddIns.Setup
 				return tmpfile;
 			}
 
-			monitor.BeginTask ("Requesting " + url, 2);
+			monitor.BeginTask (GettextCatalog.GetString ("Requesting ") + url, 2);
 			HttpWebRequest req = (HttpWebRequest) WebRequest.Create (url);
 			req.Headers ["Pragma"] = "no-cache";
 			HttpWebResponse resp = (HttpWebResponse) req.GetResponse ();
 			monitor.Step (1);
 			
-			monitor.BeginTask ("Downloading " + url, (int) resp.ContentLength);
+			monitor.BeginTask (GettextCatalog.GetString ("Downloading ") + url, (int) resp.ContentLength);
 			
 			string file = Path.GetTempFileName ();
 			FileStream fs = null;
@@ -1118,7 +1157,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 		
 		internal static string GetUninstallErrorNoRoot (AddinInfo ainfo)
 		{
-			return string.Format (GettextCatalog.GetString ("The addin '{0} v{1}' can't be uninstalled with the current user permissions."), ainfo.Id, ainfo.Version);
+			return string.Format (GettextCatalog.GetString ("The add-in '{0} v{1}' can't be uninstalled with the current user permissions."), ainfo.Id, ainfo.Version);
 		}
 	}
 	
