@@ -26,6 +26,10 @@ using Gtk;
 
 namespace Gdl
 {
+	/// <summary>
+	/// This class is used to represent a number of iconified DockItems.
+	/// It shows the icons by means of an according number of DockBarButtons.
+	/// </summary>
 	public class DockBar : VBox
 	{
 		DockMaster master;
@@ -39,7 +43,8 @@ namespace Gdl
 			Master = dock.Master;
 		}
 		
-		public DockMaster Master {
+		public DockMaster Master 
+		{
 			get { return master; }
 			set { this.Attach (value); }
 		}
@@ -56,8 +61,8 @@ namespace Gdl
 
 			// create a button for the item
 			DockBarButton button = new DockBarButton (item);
-			this.PackStart (button, false, false, 0);
-			tooltips.SetTip (button, item.Name, item.Name);
+			this.PackStart (button, false, false, 0);						
+			tooltips.SetTip (button, item.LongName, item.LongName);
 			item.DockBar = this;
 			item.DockBarButton = button;
 			button.Clicked += new EventHandler (OnDockButtonClicked);
