@@ -23,6 +23,7 @@
  */
 using System;
 using System.Xml;
+using Mono.Unix;
 using Gtk;
 
 namespace Gdl
@@ -47,7 +48,7 @@ namespace Gdl
 		private Widget tabLabel = null;
 		private int preferredWidth = -1;
 		private int preferredHeight = -1;
-		private DockPlaceholder ph = null;
+		private DockPlaceholder dockPlaceHolder = null;
 		private int startX;
 		private int startY;
 		
@@ -150,8 +151,20 @@ namespace Gdl
 			get { return !NoGrip; }
 		}
 		
-		public bool Iconified {
+		public bool Iconified
+		{
 			get { return ((DockObjectFlags & DockObjectFlags.Iconified) != 0); }
+			set
+			{ 
+				if (value)
+				{
+					DockObjectFlags |= DockObjectFlags.Iconified;
+				}
+				else
+				{
+					DockObjectFlags &= ~(DockObjectFlags.Iconified);
+				}
+			}
 		}
 		
 		public bool InDrag {
@@ -457,8 +470,8 @@ namespace Gdl
 				Remove (grip);
 				grip = null;
 			}
-			if (ph != null) {
-				ph = null;
+			if (dockPlaceHolder != null) {
+				dockPlaceHolder = null;
 			}
 			base.OnDestroyed ();
 		}
@@ -757,12 +770,12 @@ namespace Gdl
 				menu.AttachToWidget (this, new MenuDetachFunc (DetachMenu));
 				
 				// Hide menuitem
-				MenuItem mitem = new MenuItem ("Hide");
+				MenuItem mitem = new MenuItem (Catalog.GetString("Hide"));
 				mitem.Activated += new EventHandler (ItemHideCb);
 				menu.Append (mitem);
 
 				// Lock menuitem
-				CheckMenuItem citem = new CheckMenuItem ("Lock");
+				CheckMenuItem citem = new CheckMenuItem (Catalog.GetString("Lock"));
 				citem.Active = this.Locked;
 				citem.Toggled += new EventHandler (ItemLockCb);
 				menu.Append (citem);
@@ -887,7 +900,7 @@ namespace Gdl
 		}
 		
 		public void HideItem ()
-		{
+		{					
 			if (!IsAttached)
 				/* already hidden/detached */
 				return;
@@ -895,7 +908,7 @@ namespace Gdl
 			/* if the object is manual, create a new placeholder to be
 			   able to restore the position later */
 			if (!IsAutomatic)
-				ph = new DockPlaceholder (this, false);
+				dockPlaceHolder = new DockPlaceholder (this, false);
 			
 			Freeze ();
 
@@ -926,9 +939,9 @@ namespace Gdl
 		{
 			DockObjectFlags &= ~(DockObjectFlags.Iconified);
 			
-			if (ph != null) {
-				ph.Add (this);
-				ph = null;
+			if (dockPlaceHolder != null) {
+				dockPlaceHolder.Add (this);
+				dockPlaceHolder = null;
 			} else if (IsBound) {
 				if (Master.Controller != null) {
 					Master.Controller.Dock (this, DockPlacement.Floating, null);
@@ -938,19 +951,19 @@ namespace Gdl
 		
 		public virtual void SetDefaultPosition (DockObject reference)
 		{
-			ph = null;
+			dockPlaceHolder = null;
 			
 			if (reference != null && reference.IsAttached) {
 				if (reference is DockPlaceholder) {
-					ph = (DockPlaceholder)reference;
+					dockPlaceHolder = (DockPlaceholder)reference;
 				} else {
-					ph = new DockPlaceholder (reference, true);
+					dockPlaceHolder = new DockPlaceholder (reference, true);
 				}
 			}
 		}
 		
 		public DockPlaceholder DefaultPosition {
-			get { return ph; }
+			get { return dockPlaceHolder; }
 		}
 
 		private bool EventInGripWindow (Gdk.Event evnt)
