@@ -75,6 +75,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 		internal Gtk.CellRendererText text_render;
 		TreeBuilderContext builderContext;
 		Hashtable callbacks = new Hashtable ();
+		bool editingText = false;
 		
 		TreePadOption[] options;
 		TreeOptions globalOptions;
@@ -204,6 +205,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 
 			text_render = new Gtk.CellRendererText ();
 			text_render.Edited += new Gtk.EditedHandler (HandleOnEdit);
+			text_render.EditingCanceled += new EventHandler (HandleOnEditCancelled);
 			
 			complete_column.PackStart (text_render, true);
 			complete_column.AddAttribute (text_render, "text", TextColumn);
@@ -603,40 +605,77 @@ namespace MonoDevelop.Ide.Gui.Pads
 		[CommandHandler (EditCommands.Rename)]
 		public void StartLabelEdit()
 		{
-			Gtk.TreeModel foo;
-			Gtk.TreeIter iter;
-			if (!tree.Selection.GetSelected (out foo, out iter))
+			if (editingText)
+				return;
+
+			TreeNodeNavigator node = (TreeNodeNavigator) GetSelectedNode ();
+			if (node == null)
 				return;
 			
-			workNode.MoveToIter (iter);
-			workNode.ExpandToNode (); //make sure the parent of the node that is being edited is expanded
-			store.SetValue (iter, TreeViewPad.TextColumn, workNode.NodeName);
+			Gtk.TreeIter iter = node.CurrentPosition._iter;
+			object dataObject = node.DataItem;
+			NodeAttributes attributes = NodeAttributes.None;
+			
+			ITreeNavigator parentNode = node.Clone ();
+			parentNode.MoveToParent ();
+			NodePosition pos = parentNode.CurrentPosition;
+			
+			foreach (NodeBuilder b in node.NodeBuilderChain) {
+				b.GetNodeAttributes (parentNode, dataObject, ref attributes);
+				parentNode.MoveToPosition (pos);
+			}
+			
+			if ((attributes & NodeAttributes.AllowRename) == 0)
+				return;
+
+			node.ExpandToNode (); //make sure the parent of the node that is being edited is expanded
+			
+			store.SetValue (iter, TreeViewPad.TextColumn, node.NodeName);
 			
 			text_render.Editable = true;
 			tree.SetCursor (store.GetPath (iter), complete_column, true);
+			
+			editingText = true;
 		}
 
 		void HandleOnEdit (object o, Gtk.EditedArgs e)
 		{
+			editingText = false;
 			text_render.Editable = false;
+			
 			Gtk.TreeIter iter;
 			if (!store.GetIterFromString (out iter, e.Path))
 				throw new Exception("Error calculating iter for path " + e.Path);
 
-			if (e.NewText == null || e.NewText.Length == 0) {
+			if (e.NewText != null && e.NewText.Length > 0) {
+				ITreeNavigator nav = new TreeNodeNavigator (this, iter);
+				NodePosition pos = nav.CurrentPosition;
+
+				NodeBuilder[] chain = (NodeBuilder[]) store.GetValue (iter, BuilderChainColumn);
+				foreach (NodeBuilder b in chain) {
+					NodeCommandHandler handler = b.CommandHandler;
+					handler.SetCurrentNode (nav);
+					handler.RenameItem (e.NewText);
+					nav.MoveToPosition (pos);
+				}
+			}
+			
+			ITreeBuilder builder = new TreeBuilder (this, iter);
+			builder.Update ();
+		}
+		
+		void HandleOnEditCancelled (object s, EventArgs args)
+		{
+			editingText = false;
+			
+			TreeNodeNavigator node = (TreeNodeNavigator) GetSelectedNode ();
+			if (node == null)
 				return;
-			}
-
-			ITreeNavigator nav = new TreeNodeNavigator (this, iter);
-			NodePosition pos = nav.CurrentPosition;
-
-			NodeBuilder[] chain = (NodeBuilder[]) store.GetValue (iter, BuilderChainColumn);
-			foreach (NodeBuilder b in chain) {
-				NodeCommandHandler handler = b.CommandHandler;
-				handler.SetCurrentNode (nav);
-				handler.RenameItem (e.NewText);
-				nav.MoveToPosition (pos);
-			}
+			
+			// Restore the original node label
+			Gtk.TreeIter iter = node.CurrentPosition._iter;
+			ITreeBuilder builder = new TreeBuilder (this, iter);
+			builder.Update ();
 		}
 		
 		public void SaveTreeState (XmlElement el)
