@@ -37,6 +37,7 @@ using System.Collections.Specialized;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Gui.Dialogs;
 using MonoDevelop.Projects.Parser;
+using MonoDevelop.Projects.CodeGeneration;
 using MonoDevelop.Components;
 using MonoDevelop.Core;
 using MonoDevelop.Core.AddIns;
@@ -67,6 +68,7 @@ namespace MonoDevelop.Ide.Gui
 		Combine  currentCombine = null;
 		Combine  openCombine    = null;
 		IParserDatabase parserDatabase;
+		CodeRefactorer refactorer;
 
 		ICompilerResult lastResult = new DefaultCompilerResult ();
 		
@@ -83,6 +85,15 @@ namespace MonoDevelop.Ide.Gui
 		public IParserDatabase ParserDatabase {
 			get { return parserDatabase; }
 		}
+		
+		public CodeRefactorer CodeRefactorer {
+			get {
+				if (refactorer == null)
+					refactorer = new CodeRefactorer (openCombine, parserDatabase);
+				return refactorer;
+			}
+		}
+
 
 		public ICompilerResult LastCompilerResult {
 			get { return lastResult; }
@@ -185,7 +196,14 @@ namespace MonoDevelop.Ide.Gui
 				Combine closedCombine = CurrentOpenCombine;
 				CurrentSelectedProject = null;
 				CurrentOpenCombine = CurrentSelectedCombine = null;
-				IdeApp.Workbench.CloseAllDocuments ();
+				refactorer = null;
+				
+				Document[] docs = new Document [IdeApp.Workbench.Documents.Count];
+				IdeApp.Workbench.Documents.CopyTo (docs, 0);
+				foreach (Document doc in docs) {
+					if (doc.HasProject)
+						doc.Close ();
+				}
 				
 				parserDatabase.Unload (closedCombine);
 
@@ -250,6 +268,7 @@ namespace MonoDevelop.Ide.Gui
 				openCombine.FileRemovedFromProject += new ProjectFileEventHandler (NotifyFileRemovedFromProject);
 				openCombine.FileRenamedInProject += new ProjectFileRenamedEventHandler (NotifyFileRenamedInProject);
 				openCombine.FileChangedInProject += new ProjectFileEventHandler (NotifyFileChangedInProject);
+				openCombine.FilePropertyChangedInProject += new ProjectFileEventHandler (NotifyFilePropertyChangedInProject);
 				openCombine.ReferenceAddedToProject += new ProjectReferenceEventHandler (NotifyReferenceAddedToProject);
 				openCombine.ReferenceRemovedFromProject += new ProjectReferenceEventHandler (NotifyReferenceRemovedFromProject);
 				
@@ -261,7 +280,7 @@ namespace MonoDevelop.Ide.Gui
 				Services.DispatchService.GuiDispatch (new StatefulMessageHandler (RestoreCombinePreferences), CurrentOpenCombine);
 				
 				SaveCombine ();
-				monitor.ReportSuccess (GettextCatalog.GetString ("Combine loaded."));
+				monitor.ReportSuccess (GettextCatalog.GetString ("Solution loaded."));
 			} catch (Exception ex) {
 				monitor.ReportError ("Load operation failed.", ex);
 			} finally {
@@ -328,7 +347,20 @@ namespace MonoDevelop.Ide.Gui
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor ();
 			try {
 				openCombine.Save (monitor);
-				monitor.ReportSuccess (GettextCatalog.GetString ("Combine saved."));
+				monitor.ReportSuccess (GettextCatalog.GetString ("Solution saved."));
+			} catch (Exception ex) {
+				monitor.ReportError (GettextCatalog.GetString ("Save failed."), ex);
+			} finally {
+				monitor.Dispose ();
+			}
+		}
+		
+		public void SaveProject (Project project)
+		{
+			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetSaveProgressMonitor ();
+			try {
+				project.Save (monitor);
+				monitor.ReportSuccess (GettextCatalog.GetString ("Project saved."));
 			} catch (Exception ex) {
 				monitor.ReportError (GettextCatalog.GetString ("Save failed."), ex);
 			} finally {
@@ -405,23 +437,22 @@ namespace MonoDevelop.Ide.Gui
 		CombineEntry CreateCombineEntry (Combine parentCombine, bool createCombine)
 		{
 			CombineEntry res = null;
-			NewProjectDialog npdlg = new NewProjectDialog (parentCombine == null);
+			string basePath = parentCombine != null ? parentCombine.BaseDirectory : null;
+			NewProjectDialog npdlg = new NewProjectDialog (parentCombine == null, createCombine, basePath);
+			if (createCombine && parentCombine != null)
+				npdlg.SelectTemplate ("MonoDevelop.BlankSolution");
+
 			if (npdlg.Run () == (int) Gtk.ResponseType.Ok) {
 				IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetLoadProgressMonitor ();
 				try {
-					if (createCombine)
-						res = parentCombine.AddEntry (npdlg.NewCombineLocation, monitor);
-					else
-						res = parentCombine.AddEntry (npdlg.NewProjectLocation, monitor);
+					res = parentCombine.AddEntry (npdlg.NewCombineEntryLocation, monitor);
 				}
 				catch {
-					Services.MessageService.ShowError (string.Format (GettextCatalog.GetString ("The file '{0}' could not be loaded."), npdlg.NewProjectLocation));
+					Services.MessageService.ShowError (string.Format (GettextCatalog.GetString ("The file '{0}' could not be loaded."), npdlg.NewCombineEntryLocation));
 					res = null;
 				}
 				monitor.Dispose ();
 			}
-			
-			npdlg = null;
 
 			if (res != null)
 				SaveCombine ();
@@ -454,35 +485,19 @@ namespace MonoDevelop.Ide.Gui
 			return res;
 		}
 		
-		public ProjectFile CreateProjectFile (Project parentProject, string basePath)
+		public void CreateProjectFile (Project parentProject, string basePath)
 		{
-			NewFileDialog nfd = new NewFileDialog ();
-			int res = nfd.Run ();
-			nfd.Dispose ();
-			if (res != (int) Gtk.ResponseType.Ok) return null;
-
-			Document doc = IdeApp.Workbench.ActiveDocument;
-			int count = 1;
-				
-			string baseName  = Path.GetFileNameWithoutExtension (doc.Window.ViewContent.UntitledName);
-			string extension = Path.GetExtension (doc.Window.ViewContent.UntitledName);
-				
-			// first try the default untitled name of the viewcontent filename
-			string fileName = Path.Combine (basePath, baseName +  extension);
-				
-			// if it is already in the project, or it does exists we try to get a name that is
-			// untitledName + Numer + extension
-			while (parentProject.IsFileInProject (fileName) || System.IO.File.Exists (fileName)) {
-				fileName = Path.Combine (basePath, baseName + count.ToString() + extension);
-				++count;
+			CreateProjectFile (parentProject, basePath, null);
+		}
+		
+		public void CreateProjectFile (Project parentProject, string basePath, string selectedTemplateId)
+		{
+			using (NewFileDialog nfd = new NewFileDialog (parentProject, basePath)) {
+				if (selectedTemplateId != null)
+					nfd.SelectTemplate (selectedTemplateId);
+				nfd.Run ();
+				nfd.Dispose ();
 			}
-
-			// now we have a valid filename which we could use
-			doc.SaveAs (fileName);
-				
-			ProjectFile newFileInformation = new ProjectFile(fileName, BuildAction.Compile);
-			parentProject.ProjectFiles.Add (newFileInformation);
-			return newFileInformation;
 		}
 
 		public bool AddReferenceToProject (Project project)
@@ -847,14 +862,17 @@ namespace MonoDevelop.Ide.Gui
 				Services.TaskService.AddTask (new Task(null, err));
 			}
 			
+			string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", result.ErrorCount, result.ErrorCount);
+			string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", result.WarningCount, result.WarningCount);
+
 			if (result.ErrorCount == 0 && result.WarningCount == 0 && lastResult.FailedBuildCount == 0) {
 				monitor.ReportSuccess (GettextCatalog.GetString ("Build successful."));
 			} else if (result.ErrorCount == 0 && result.WarningCount > 0) {
-				monitor.ReportWarning (String.Format (GettextCatalog.GetString ("Build: {0} errors, {1} warnings."), result.ErrorCount, result.WarningCount));
+				monitor.ReportWarning(GettextCatalog.GetString("Build: ") + errorString + ", " + warningString);
 			} else if (result.ErrorCount > 0) {
-				monitor.ReportError (String.Format (GettextCatalog.GetString ("Build: {0} errors, {1} warnings."), result.ErrorCount, result.WarningCount), null);
+				monitor.ReportError(GettextCatalog.GetString("Build: ") + errorString + ", " + warningString, null);
 			} else {
-				monitor.ReportError (String.Format (GettextCatalog.GetString ("Build failed.")), null);
+				monitor.ReportError(GettextCatalog.GetString("Build failed."), null);
 			}
 			
 			OnEndBuild (lastResult.FailedBuildCount == 0);
@@ -1005,6 +1023,11 @@ namespace MonoDevelop.Ide.Gui
 			OnFileChangedInProject (e);
 		}		
 		
+		internal void NotifyFilePropertyChangedInProject (object sender, ProjectFileEventArgs e)
+		{
+			OnFilePropertyChangedInProject (e);
+		}		
+		
 		internal void NotifyReferenceAddedToProject (object sender, ProjectReferenceEventArgs e)
 		{
 			OnReferenceAddedToProject (e);
@@ -1043,6 +1066,13 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
+		protected virtual void OnFilePropertyChangedInProject (ProjectFileEventArgs e)
+		{
+			if (FilePropertyChangedInProject != null) {
+				FilePropertyChangedInProject (this, e);
+			}
+		}
+		
 		protected virtual void OnReferenceRemovedFromProject (ProjectReferenceEventArgs e)
 		{
 			if (ReferenceRemovedFromProject != null) {
@@ -1112,6 +1142,7 @@ namespace MonoDevelop.Ide.Gui
 		public event ProjectFileEventHandler FileRemovedFromProject;
 		public event ProjectFileEventHandler FileAddedToProject;
 		public event ProjectFileEventHandler FileChangedInProject;
+		public event ProjectFileEventHandler FilePropertyChangedInProject;
 		public event ProjectFileRenamedEventHandler FileRenamedInProject;
 		
 		public event EventHandler StartBuild;
