@@ -56,6 +56,8 @@ namespace MonoDevelop.Ide.Gui
 		
 		WorkbenchContextCodon[] contextCodons;
 		bool initialized;
+		IWorkbenchWindow lastActive;
+		bool ignorePageSwitch;
 
 		public SdiWorkbenchLayout () {
 			contextChangedHandler = new EventHandler (OnContextChanged);
@@ -227,12 +229,13 @@ namespace MonoDevelop.Ide.Gui
 				}
 				else
 				{
-					if (currentLayout == "")
-						// if the layout doesn't exists and we need to
+					if (currentLayout == "") {
+						// if the layout doesn't exist and we need to
 						// load a layout (ie.  we've just been
 						// created), load the default so old layout
 						// xml files work smoothly
 						dockLayout.LoadLayout (null);
+					}
 					
 					// the layout didn't exist, so save it and add it to our list
 					dockLayout.SaveLayout (newLayout);
@@ -413,6 +416,11 @@ namespace MonoDevelop.Ide.Gui
 			if (placement != null)
 				GetPlacement (placement, out dockPlacement, out ot);
 				
+			if (item.Iconified)
+			{
+				item.Iconified = false;
+			}
+			
 			if (dockPlacement != DockPlacement.None && dockPlacement != DockPlacement.Floating) {
 				if (ot != null) {
 					item.DockTo (ot, dockPlacement);
@@ -506,7 +514,7 @@ namespace MonoDevelop.Ide.Gui
 		public IWorkbenchWindow ShowView(IViewContent content)
 		{	
 			Gtk.Image mimeimage = null;
-			if (content.IsUntitled) {
+			if (content.IsUntitled && content.UntitledName == null) {
 				mimeimage = new Gtk.Image (FileIconLoader.GetPixbufForType ("gnome-fs-regular", 16));
 			} else {
 				mimeimage = new Gtk.Image (FileIconLoader.GetPixbufForFile (content.ContentName, 16));
@@ -520,7 +528,7 @@ namespace MonoDevelop.Ide.Gui
 
 			sdiWorkspaceWindow.Closed += new EventHandler(CloseWindowEvent);
 			tabControl.InsertPage (sdiWorkspaceWindow, tabLabel, -1);
-		
+			
 			tabLabel.Show();
 			return sdiWorkspaceWindow;
 		}
@@ -547,7 +555,17 @@ namespace MonoDevelop.Ide.Gui
 		}
 
 		public void RemoveTab (int pageNum) {
-			tabControl.RemovePage (pageNum);
+			try {
+				// Weird switch page events are fired when a tab is removed.
+				// This flag avoids unneeded events.
+				ignorePageSwitch = true;
+				IWorkbenchWindow w = ActiveWorkbenchwindow;
+				tabControl.RemovePage (pageNum);
+				if (w != ActiveWorkbenchwindow)
+					ActiveMdiChanged (null, null);
+			} finally {
+				ignorePageSwitch = false;
+			}
 		}
 
 		/// <summary>
@@ -568,6 +586,14 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void ActiveMdiChanged(object sender, SwitchPageArgs e)
 		{
+			if (ignorePageSwitch)
+				return;
+
+			if (lastActive == ActiveWorkbenchwindow)
+				return;
+				
+			lastActive = ActiveWorkbenchwindow;
+
 			try {
 				if (ActiveWorkbenchwindow != null) {
 					if (ActiveWorkbenchwindow.ViewContent.IsUntitled) {
@@ -577,7 +603,7 @@ namespace MonoDevelop.Ide.Gui
 						if (ActiveWorkbenchwindow.ViewContent.IsDirty) {
 							post = "*";
 						}
-						if (ActiveWorkbenchwindow.ViewContent.HasProject)
+						if (ActiveWorkbenchwindow.ViewContent.Project != null)
 						{
 							((Gtk.Window)workbench).Title = ActiveWorkbenchwindow.ViewContent.Project.Name + " - " + ActiveWorkbenchwindow.ViewContent.PathRelativeToProject + post + " - MonoDevelop";
 						}
