@@ -25,16 +25,14 @@ using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Core.AddIns;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Utility;
+using MonoDevelop.Projects.Text;
 
 namespace MonoDevelop.Projects.Parser
 {
 	public class DefaultParserService : AbstractService, IParserService
 	{
-		IParser[] parsers;
-		
 		public override void InitializeService()
 		{
-			parsers = (IParser[]) Runtime.AddInService.GetTreeItems ("/Workspace/Parser", typeof(IParser));
 		}
 		
 		public IParserDatabase CreateParserDatabase ()
@@ -48,31 +46,52 @@ namespace MonoDevelop.Projects.Parser
 		{
 			IParser parser = GetParser (fileName);
 			if (parser != null) {
-				return parser.ExpressionFinder;
+				return parser.CreateExpressionFinder (fileName);
 			}
 			return null;
 		}
 		
 		public virtual IParser GetParser (string fileName)
 		{
-			// HACK: I'm too lazy to do it 'right'
-			// HACK: Still a hack, but extensible
-			if (fileName != null) {
-				foreach(IParser p in parsers) {
-					if (p.CanParse(fileName)) {
-						return p;
-					}
-				}
-			}
-			return null;
+			return Services.Languages.GetParserForFile (fileName);
 		}
 		
-		public void GenerateAssemblyDatabase (string baseDir, string name)
+		public string GenerateAssemblyDatabase (string baseDir, string name)
 		{
 			AssemblyCodeCompletionDatabase db = new AssemblyCodeCompletionDatabase (baseDir, name, (ParserDatabase) CreateParserDatabase());
 			db.ParseInExternalProcess = false;
 			db.ParseAll ();
 			db.Write ();
+			return db.DataFile;
+		}
+	}
+	
+	internal class FileParserContext: ParserContext, IFileParserContext
+	{
+		string file;
+		
+		public FileParserContext (DefaultParserService parserService, ParserDatabase pdb, CodeCompletionDatabase db, string file): base (parserService, pdb, db)
+		{
+			this.file = file;
+		}
+		
+		public IParseInformation ParseFile ()
+		{
+			return ParseFile (file);
+		}
+	}
+	
+	internal class ProjectParserContext: ParserContext, IProjectParserContext
+	{
+		public ProjectParserContext (DefaultParserService parserService, ParserDatabase pdb, CodeCompletionDatabase db): base (parserService, pdb, db)
+		{
+		}
+	}
+	
+	internal class AssemblyParserContext: ParserContext, IAssemblyParserContext
+	{
+		public AssemblyParserContext (DefaultParserService parserService, ParserDatabase pdb, CodeCompletionDatabase db): base (parserService, pdb, db)
+		{
 		}
 	}
 	
@@ -104,6 +123,11 @@ namespace MonoDevelop.Projects.Parser
 			return pdb.ParseFile (fileName, fileContent);
 		}
 		
+		public IParseInformation ParseFile (ITextFile file)
+		{
+			return pdb.ParseFile (file.Name, file.Text);
+		}
+		
 		public IParseInformation GetParseInformation (string fileName)
 		{
 			return pdb.GetParseInformation (fileName);
@@ -124,7 +148,7 @@ namespace MonoDevelop.Projects.Parser
 			return pdb.GetNamespaceList (db, subNameSpace);
 		}
 		
-		public ArrayList GetNamespaceContents (string subNameSpace, bool includeReferences)
+		public LanguageItemCollection GetNamespaceContents (string subNameSpace, bool includeReferences)
 		{
 			return pdb.GetNamespaceContents (db, subNameSpace, includeReferences);
 		}
@@ -160,7 +184,7 @@ namespace MonoDevelop.Projects.Parser
 			return pdb.GetNamespaceList (db, subNameSpace, includeReferences, caseSensitive);
 		}
 		
-		public ArrayList GetNamespaceContents (string subNameSpace, bool includeReferences, bool caseSensitive)
+		public LanguageItemCollection GetNamespaceContents (string subNameSpace, bool includeReferences, bool caseSensitive)
 		{
 			return pdb.GetNamespaceContents (db, subNameSpace, includeReferences, caseSensitive);
 		}
@@ -215,21 +239,8 @@ namespace MonoDevelop.Projects.Parser
 				return null;
 			}
 		}
-		
-		public string MonodocResolver (string expression, int caretLineNumber, int caretColumn, string fileName, string fileContent)
-		{
-			try {
-				IParser parser = parserService.GetParser (fileName);
-				if (parser != null) {
-					return parser.MonodocResolver (this, expression, caretLineNumber, caretColumn, fileName, fileContent);
-				}
-				return null;
-			} catch {
-				return null;
-			}
-		}
-		
-		public ArrayList IsAsResolve (string expression, int caretLineNumber, int caretColumn, string fileName, string fileContent)
+			
+		public LanguageItemCollection IsAsResolve (string expression, int caretLineNumber, int caretColumn, string fileName, string fileContent)
 		{
 			try {
 				IParser parser = parserService.GetParser (fileName);
@@ -242,7 +253,7 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
-		public ArrayList CtrlSpace (int caretLine, int caretColumn, string fileName)
+		public LanguageItemCollection CtrlSpace (int caretLine, int caretColumn, string fileName)
 		{
 			IParser parser = parserService.GetParser (fileName);
 			if (parser != null) {
@@ -250,6 +261,57 @@ namespace MonoDevelop.Projects.Parser
 			}
 			return null;
 		}
+		
+		public ILanguageItem ResolveIdentifier (string id, int caretLineNumber, int caretColumn, string fileName, string fileContent)
+		{
+			try {
+				IParser parser = parserService.GetParser (fileName);
+				if (parser != null) {
+					return parser.ResolveIdentifier (this, id, caretLineNumber, caretColumn, fileName, fileContent);
+				}
+				return null;
+			} catch {
+				return null;
+			}
+		}
+		
+		public ILanguageItem GetEnclosingLanguageItem (int caretLineNumber, int caretColumn, ITextFile file)
+		{
+			IClass[] classes = GetFileContents (file.Name);			
+			IClass cls = GetEnclosingClass (caretLineNumber, caretColumn, classes);
+			if (cls == null)
+				return null;
+
+			foreach (IField f in cls.Fields)
+				if (f.Region != null && f.Region.IsInside (caretLineNumber, caretColumn))
+					return f;
+			foreach (IMethod m in cls.Methods)
+				if ((m.Region != null && (m.Region.IsInside (caretLineNumber, caretColumn)) || (m.BodyRegion != null && m.BodyRegion.IsInside (caretLineNumber, caretColumn))))
+					return m;
+			foreach (IProperty m in cls.Properties)
+				if ((m.Region != null && (m.Region.IsInside (caretLineNumber, caretColumn)) || (m.BodyRegion != null && m.BodyRegion.IsInside (caretLineNumber, caretColumn))))
+					return m;
+			foreach (IEvent m in cls.Events)
+				if (m.Region != null && m.Region.IsInside (caretLineNumber, caretColumn))
+					return m;
+			foreach (IIndexer m in cls.Indexer)
+				if (m.Region != null && m.Region.IsInside (caretLineNumber, caretColumn))
+					return m;
+			return null;
+		}
+		
+		IClass GetEnclosingClass (int line, int col, IEnumerable classes)
+		{
+			foreach (IClass cls in classes) {
+				if (cls.InnerClasses != null) {
+					IClass c =  GetEnclosingClass (line, col, cls.InnerClasses);
+					if (c != null) return c;
+				}
+				if (cls.Region.IsInside (line, col) || (cls.BodyRegion != null && cls.BodyRegion.IsInside (line, col)))
+					return cls;
+			}
+			return null;
+		}
 	}
 	
 	internal class ParserDatabase : IParserDatabase
@@ -260,6 +322,10 @@ namespace MonoDevelop.Projects.Parser
 		bool trackingFileChanges;
 		IProgressMonitorFactory parseProgressMonitorFactory;
 		
+		// Only keeps track of explicitely loaded assemblies, not the ones
+		// referenced by projects.
+		Hashtable loadedAssemblies = new Hashtable ();
+		
 		const int MAX_PARSING_CACHE_SIZE = 10;
 		const int MAX_SINGLEDB_CACHE_SIZE = 10;
 		string CoreDB;
@@ -376,14 +442,19 @@ namespace MonoDevelop.Projects.Parser
 			nameTable = new StringNameTable (sharedNameTable);
 		}
 		
-		public IParserContext GetProjectParserContext (Project project)
+		public IProjectParserContext GetProjectParserContext (Project project)
 		{
-			return new ParserContext (parserService, this, GetProjectDatabase (project));
+			return new ProjectParserContext (parserService, this, GetProjectDatabase (project));
 		}
 		
-		public IParserContext GetFileParserContext (string file)
+		public IFileParserContext GetFileParserContext (string file)
 		{
-			return new ParserContext (parserService, this, GetSingleFileDatabase (file));
+			return new FileParserContext (parserService, this, GetSingleFileDatabase (file), file);
+		}
+		
+		public IAssemblyParserContext GetAssemblyParserContext (string assemblyFile)
+		{
+			return new AssemblyParserContext (parserService, this, GetAssemblyDatabase (assemblyFile));
 		}
 		
 		public IProgressMonitorFactory ParseProgressMonitorFactory {
@@ -488,6 +559,11 @@ namespace MonoDevelop.Projects.Parser
 			return (ProjectCodeCompletionDatabase) GetDatabase (null, "Project:" + project.Name);
 		}
 		
+		internal CodeCompletionDatabase GetAssemblyDatabase (string assemblyName)
+		{
+			return GetDatabase (null, "Assembly:" + assemblyName);
+		}
+		
 		internal CodeCompletionDatabase GetDatabase (string baseDir, string uri)
 		{
 			lock (databases)
@@ -563,6 +639,38 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
+		public string LoadAssembly (string assemblyName)
+		{
+			string aname = AssemblyCodeCompletionDatabase.GetFullAssemblyName (assemblyName);
+			string name = "Assembly:" + aname;
+			
+			lock (databases) {
+				object c = loadedAssemblies [name];
+				if (c == null)
+					loadedAssemblies [name] = 1;
+				else
+					loadedAssemblies [name] = ((int)c) + 1;
+			}
+			return aname;
+		}
+		
+		public void UnloadAssembly (string assemblyName)
+		{
+			string name = "Assembly:" + AssemblyCodeCompletionDatabase.GetFullAssemblyName (assemblyName);
+			
+			lock (databases) {
+				object c = loadedAssemblies [name];
+				if (c != null) {
+					int nc = ((int)c) - 1;
+					if (nc == 0)
+						loadedAssemblies.Remove (name);
+					else
+						loadedAssemblies [name] = nc;
+				}
+			}
+			CleanUnusedDatabases ();
+		}
+		
 		public void Load (CombineEntry entry)
 		{
 			if (entry is Project)
@@ -640,7 +748,10 @@ namespace MonoDevelop.Projects.Parser
 				ArrayList todel = new ArrayList ();
 				foreach (DictionaryEntry en in databases)
 				{
-					if (!(en.Value is ProjectCodeCompletionDatabase) && !references.Contains (en.Key))
+					if (!(en.Value is ProjectCodeCompletionDatabase) &&
+						!references.Contains (en.Key) &&
+						!loadedAssemblies.Contains (en.Key)
+					)
 						todel.Add (en.Key);
 				}
 				
@@ -875,18 +986,25 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
-		public void UpdateFile (Project project, string fileName, string fileContent)
+		public IParseInformation UpdateFile (Project project, string fileName, string fileContent)
 		{
 			try {
 				if (parserService.GetParser (fileName) == null)
-					return;
+					return null;
 				
 				IParseInformation parseInformation = null;
+				if (fileContent == null) {
+					StreamReader sr = new StreamReader (fileName);
+					fileContent = sr.ReadToEnd ();
+					sr.Close ();
+				}
+
 				int contentHash = fileContent.GetHashCode ();
 			
 				if (lastUpdateSize[fileName] == null || (int)lastUpdateSize[fileName] != contentHash) {
 					parseInformation = DoParseFile (fileName, fileContent);
-					if (parseInformation == null) return;
+					if (parseInformation == null)
+						return null;
 					
 					if (project != null) {
 						ProjectCodeCompletionDatabase db = GetProjectDatabase (project);
@@ -899,9 +1017,12 @@ namespace MonoDevelop.Projects.Parser
 					}
 
 					lastUpdateSize[fileName] = contentHash;
-				}
+					return parseInformation;
+				} else
+					return this.GetCachedParseInformation (fileName);
 			} catch (Exception e) {
 				Runtime.LoggingService.Error (e.ToString ());
+				return null;
 			}
 		}
 		
@@ -1032,14 +1153,14 @@ namespace MonoDevelop.Projects.Parser
 			return (string[]) contents.ToArray (typeof(string));
 		}
 		
-		public ArrayList GetNamespaceContents (CodeCompletionDatabase db, string namspace, bool includeReferences)
+		public LanguageItemCollection GetNamespaceContents (CodeCompletionDatabase db, string namspace, bool includeReferences)
 		{
 			return GetNamespaceContents (db, namspace, includeReferences, true);
 		}
 		
-		public ArrayList GetNamespaceContents (CodeCompletionDatabase db, string namspace, bool includeReferences, bool caseSensitive)
+		public LanguageItemCollection GetNamespaceContents (CodeCompletionDatabase db, string namspace, bool includeReferences, bool caseSensitive)
 		{
-			ArrayList contents = new ArrayList ();
+			LanguageItemCollection contents = new LanguageItemCollection ();
 			
 			if (db != null) {
 				db.GetNamespaceContents (contents, namspace, caseSensitive);
@@ -1391,6 +1512,13 @@ namespace MonoDevelop.Projects.Parser
 			OnClassInformationChanged (args);
 		}
 
+		public void NotifyAssemblyInfoChange (string file, string asmName)
+		{
+			AssemblyInformationEventArgs args = new AssemblyInformationEventArgs (file, asmName);
+			if (AssemblyInformationChanged != null)
+				AssemblyInformationChanged (this, args);
+		}
+
 		protected virtual void OnParseInformationChanged(ParseInformationEventArgs e)
 		{
 			if (ParseInformationChanged != null) {
@@ -1407,6 +1535,7 @@ namespace MonoDevelop.Projects.Parser
 		
 		public event ParseInformationEventHandler ParseInformationChanged;
 		public event ClassInformationEventHandler ClassInformationChanged;
+		public event AssemblyInformationEventHandler AssemblyInformationChanged;
 	}
 	
 	[Serializable]
