@@ -17,7 +17,7 @@ using System.ComponentModel;
 using System.Threading;
 
 using Mono.Unix.Native;
-using FileMode = Mono.Unix.FilePermissions;
+using FileMode = Mono.Unix.Native.FilePermissions;
 
 using MonoDevelop.Core;
 using MonoDevelop.Core.Properties;
@@ -54,6 +54,7 @@ namespace MonoDevelop.Projects
 		ProjectFileEventHandler fileAddedToProjectHandler;
 		ProjectFileEventHandler fileRemovedFromProjectHandler;
 		ProjectFileEventHandler fileChangedInProjectHandler;
+		ProjectFileEventHandler filePropertyChangedInProjectHandler;
 		ProjectFileRenamedEventHandler fileRenamedInProjectHandler;
 
 		ProjectReferenceEventHandler referenceAddedToProjectHandler;
@@ -141,6 +142,7 @@ namespace MonoDevelop.Projects
 		{
 			fileAddedToProjectHandler = new ProjectFileEventHandler (NotifyFileAddedToProject);
 			fileChangedInProjectHandler = new ProjectFileEventHandler (NotifyFileChangedInProject);
+			filePropertyChangedInProjectHandler = new ProjectFileEventHandler (NotifyFilePropertyChangedInProject);
 			fileRemovedFromProjectHandler = new ProjectFileEventHandler (NotifyFileRemovedFromProject);
 			fileRenamedInProjectHandler = new ProjectFileRenamedEventHandler (NotifyFileRenamedInProject);
 			referenceAddedToProjectHandler = new ProjectReferenceEventHandler (NotifyReferenceAddedToProject);
@@ -155,14 +157,21 @@ namespace MonoDevelop.Projects
 		}
 		
 		protected override void OnActiveConfigurationChanged (ConfigurationEventArgs args)
+		{
+			UpdateActiveConfigurationTree ();
+			base.OnActiveConfigurationChanged  (args);
+		}
+		
+		internal void UpdateActiveConfigurationTree ()
 		{
 			if (ActiveConfiguration != null && !deserializing) {
 				foreach (CombineConfigurationEntry cce in ((CombineConfiguration)ActiveConfiguration).Entries) {
-					IConfiguration conf = cce.Entry.GetConfiguration (cce.ConfigurationName);
-					cce.Entry.ActiveConfiguration = conf;
+					if (cce.Entry != null) {
+						IConfiguration conf = cce.Entry.GetConfiguration (cce.ConfigurationName);
+						cce.Entry.ActiveConfiguration = conf;
+					}
 				}
 			}
-			base.OnActiveConfigurationChanged  (args);
 		}
 		
 		internal void NotifyEntryAdded (CombineEntry entry)
@@ -180,10 +189,14 @@ namespace MonoDevelop.Projects
 						ActiveConfiguration = cconf;
 				}
 			}
-			
+		
 			foreach (CombineConfiguration conf in Configurations)
-				conf.AddEntry (entry);
-
+				conf.AddEntry (entry);						
+			
+			if (entry.Configurations.Count == 0 && entry is Combine)			
+				foreach (CombineConfiguration conf in Configurations)					
+					entry.Configurations.Add(new CombineConfiguration(conf.Name));			
+			
 			combineExecuteDefinitions.Add (new CombineExecuteDefinition (entry, EntryExecuteType.None));
 			
 			if (eventsAllowed)
@@ -195,6 +208,7 @@ namespace MonoDevelop.Projects
 				project.FileRemovedFromProject += fileRemovedFromProjectHandler;
 				project.FileAddedToProject += fileAddedToProjectHandler;
 				project.FileChangedInProject += fileChangedInProjectHandler;
+				project.FilePropertyChangedInProject += filePropertyChangedInProjectHandler;
 				project.FileRenamedInProject += fileRenamedInProjectHandler;
 				project.ReferenceRemovedFromProject += referenceRemovedFromProjectHandler;
 				project.ReferenceAddedToProject += referenceAddedToProjectHandler;
@@ -205,6 +219,7 @@ namespace MonoDevelop.Projects
 				combine.FileRemovedFromProject += fileRemovedFromProjectHandler;
 				combine.FileAddedToProject += fileAddedToProjectHandler;
 				combine.FileChangedInProject += fileChangedInProjectHandler;
+				combine.FilePropertyChangedInProject += filePropertyChangedInProjectHandler;
 				combine.FileRenamedInProject += fileRenamedInProjectHandler;
 				combine.ReferenceRemovedFromProject += referenceRemovedFromProjectHandler;
 				combine.ReferenceAddedToProject += referenceAddedToProjectHandler;
@@ -238,6 +253,7 @@ namespace MonoDevelop.Projects
 					conf.SetCombine (this);
 			} finally {
 				deserializing = false;
+				UpdateActiveConfigurationTree ();
 			}
 		}
 
@@ -267,6 +283,7 @@ namespace MonoDevelop.Projects
 				pce.FileRemovedFromProject -= fileRemovedFromProjectHandler;
 				pce.FileAddedToProject -= fileAddedToProjectHandler;
 				pce.FileChangedInProject -= fileChangedInProjectHandler;
+				pce.FilePropertyChangedInProject -= filePropertyChangedInProjectHandler;
 				pce.FileRenamedInProject -= fileRenamedInProjectHandler;
 				pce.ReferenceRemovedFromProject -= referenceRemovedFromProjectHandler;
 				pce.ReferenceAddedToProject -= referenceAddedToProjectHandler;
@@ -277,6 +294,7 @@ namespace MonoDevelop.Projects
 					cce.FileRemovedFromProject -= fileRemovedFromProjectHandler;
 					cce.FileAddedToProject -= fileAddedToProjectHandler;
 					cce.FileChangedInProject -= fileChangedInProjectHandler;
+					cce.FilePropertyChangedInProject -= filePropertyChangedInProjectHandler;
 					cce.FileRenamedInProject -= fileRenamedInProjectHandler;
 					cce.ReferenceRemovedFromProject -= referenceRemovedFromProjectHandler;
 					cce.ReferenceAddedToProject -= referenceAddedToProjectHandler;
@@ -431,10 +449,20 @@ namespace MonoDevelop.Projects
 		/// undercombines
 		/// </remarks>
 		public CombineEntryCollection GetAllProjects ()
+		{
+			return GetAllProjects (false);
+		}
+		
+		// When topologicalSort=true, the projects are returned in the order
+		// they should be compiled, acording to their references.
+		public CombineEntryCollection GetAllProjects (bool topologicalSort)
 		{
 			CombineEntryCollection list = new CombineEntryCollection();
 			GetAllProjects (list);
-			return list;
+			if (topologicalSort)
+				return TopologicalSort (list);
+			else
+				return list;
 		}
 		
 		void GetAllProjects (CombineEntryCollection list)
@@ -760,6 +788,11 @@ namespace MonoDevelop.Projects
 			OnFileChangedInProject (e);
 		}
 		
+		internal void NotifyFilePropertyChangedInProject (object sender, ProjectFileEventArgs e)
+		{
+			OnFilePropertyChangedInProject (e);
+		}
+		
 		internal void NotifyFileRenamedInProject (object sender, ProjectFileRenamedEventArgs e)
 		{
 			OnFileRenamedInProject (e);
@@ -810,6 +843,13 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		protected virtual void OnFilePropertyChangedInProject (ProjectFileEventArgs e)
+		{
+			if (FilePropertyChangedInProject != null) {
+				FilePropertyChangedInProject (this, e);
+			}
+		}
+		
 		protected virtual void OnFileAddedToProject (ProjectFileEventArgs e)
 		{
 			if (FileAddedToProject != null) {
@@ -844,6 +884,7 @@ namespace MonoDevelop.Projects
 		public event ProjectFileEventHandler FileAddedToProject;
 		public event ProjectFileEventHandler FileRemovedFromProject;
 		public event ProjectFileEventHandler FileChangedInProject;
+		public event ProjectFileEventHandler FilePropertyChangedInProject;
 		public event ProjectFileRenamedEventHandler FileRenamedInProject;
 		public event ProjectReferenceEventHandler ReferenceAddedToProject;
 		public event ProjectReferenceEventHandler ReferenceRemovedFromProject;
