@@ -52,6 +52,8 @@ namespace MonoDevelop.Projects
 		[ItemProperty ("DeploymentInformation")]
 		protected DeployInformation deployInformation = new DeployInformation();
 		
+		IBuildStep[] buildPipeline;
+		
 		bool isDirty = false;
 		bool filesChecked;
 		
@@ -301,25 +303,35 @@ namespace MonoDevelop.Projects
 				return new DefaultCompilerResult (new CompilerResults (null), "");
 			
 			try {
-				monitor.BeginTask (String.Format (GettextCatalog.GetString ("Building Project: {0} Configuration: {1}"), Name, ActiveConfiguration.Name), 3);
+			
+				IBuildStep[] steps = BuildPipeline;
+				
+				monitor.BeginTask (String.Format (GettextCatalog.GetString ("Building Project: {0} Configuration: {1}"), Name, ActiveConfiguration.Name), steps.Length);
 				
 				Runtime.StringParserService.Properties["Project"] = Name;
 				
-				DoPreBuild (monitor);
+				ICompilerResult res = null;
 				
-				monitor.Step (1);
-				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Performing main compilation...")));
-				
-				ICompilerResult res = DoBuild (monitor);
-				
-				monitor.Step (1);
-				
-				DoPostBuild (monitor);
+				foreach (IBuildStep step in BuildPipeline) {
+					ICompilerResult sres = step.Build (monitor, this);
+					if (sres != null) {
+						if (res != null) {
+							CompilerResults cres = new CompilerResults (null);
+							cres.Errors.AddRange (res.CompilerResults.Errors);
+							cres.Errors.AddRange (sres.CompilerResults.Errors);
+							res = new DefaultCompilerResult (cres, res.CompilerOutput + "\n" + sres.CompilerOutput);
+						} else
+							res = sres;
+					}
+					monitor.Step (1);
+				}
 				
 				isDirty = false;
 				
-				monitor.Step (1);
-				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Build complete -- {0} errors, {1} warnings"), res.ErrorCount, res.WarningCount));
+				string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", res.ErrorCount, res.ErrorCount);
+				string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", res.WarningCount, res.WarningCount);
+				
+				monitor.Log.WriteLine(GettextCatalog.GetString("Build complete -- ") + errorString + ", " + warningString);
 				
 				return res;
 			} finally {
@@ -327,6 +339,15 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		protected virtual IBuildStep[] BuildPipeline {
+			get {
+				if (buildPipeline == null)
+					buildPipeline = (IBuildStep[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/BuildPipeline", typeof(IBuildStep));
+				return buildPipeline;
+			}
+		}
+		
+		
 		void GetReferencedProjects (CombineEntryCollection referenced, Project project)
 		{
 			if (referenced.Contains (project)) return;
@@ -513,6 +534,11 @@ namespace MonoDevelop.Projects
 			OnFileChangedInProject (new ProjectFileEventArgs (this, file));
 		}
 		
+ 		internal void NotifyFilePropertyChangedInProject (ProjectFile file)
+		{
+			OnFilePropertyChangedInProject (new ProjectFileEventArgs (this, file));
+		}
+		
 		internal void NotifyFileRemovedFromProject (ProjectFile file)
 		{
 			isDirty = true;
@@ -578,6 +604,13 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+ 		protected virtual void OnFilePropertyChangedInProject (ProjectFileEventArgs e)
+		{
+			if (FilePropertyChangedInProject != null) {
+				FilePropertyChangedInProject (this, e);
+			}
+		}
+		
  		protected virtual void OnFileRenamedInProject (ProjectFileRenamedEventArgs e)
 		{
 			if (FileRenamedInProject != null) {
@@ -588,56 +621,52 @@ namespace MonoDevelop.Projects
 		public event ProjectFileEventHandler FileRemovedFromProject;
 		public event ProjectFileEventHandler FileAddedToProject;
 		public event ProjectFileEventHandler FileChangedInProject;
+		public event ProjectFileEventHandler FilePropertyChangedInProject;
 		public event ProjectFileRenamedEventHandler FileRenamedInProject;
 		public event ProjectReferenceEventHandler ReferenceRemovedFromProject;
 		public event ProjectReferenceEventHandler ReferenceAddedToProject;
-	}
-	
-	public class ProjectActiveConfigurationTypeConverter : TypeConverter
-	{
-		public override bool CanConvertFrom(ITypeDescriptorContext context,Type sourceType)
-		{
-			return true;
-		}
 		
-		public override  bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
+		class MainBuildStep: IBuildStep
 		{
-			return true;
-		}
-		
-		public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture,  object value)
-		{
-			Project project = (Project)context.Instance;
-			foreach (IConfiguration configuration in project.Configurations) {
-				if (configuration.Name == value.ToString()) {
-					return configuration;
-				}
+			public ICompilerResult Build (IProgressMonitor monitor, Project project)
+			{
+				monitor.Log.WriteLine (String.Format (GettextCatalog.GetString ("Performing main compilation...")));
+				return project.DoBuild (monitor);
 			}
-			return null;
-		}
-		
-		public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
-		{
-			IConfiguration config = value as IConfiguration;
-			Debug.Assert(config != null, String.Format("Tried to convert {0} to IConfiguration", config));
-			if (config != null) {
-				return config.Name;
+			
+			public bool NeedsBuilding (Project project)
+			{
+				if (!project.isDirty) project.CheckNeedsBuild ();
+				return project.isDirty;
 			}
-			return String.Empty;
 		}
 		
-		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
+		class PreBuildStep: IBuildStep
 		{
-			return true;
-		}
-		public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
-		{
-			return true;
+			public ICompilerResult Build (IProgressMonitor monitor, Project project)
+			{
+				project.DoPreBuild (monitor);
+				return null;
+			}
+			
+			public bool NeedsBuilding (Project project)
+			{
+				return false;
+			}
 		}
 		
-		public override System.ComponentModel.TypeConverter.StandardValuesCollection GetStandardValues(System.ComponentModel.ITypeDescriptorContext context)
+		class PostBuildStep: IBuildStep
 		{
-			return new TypeConverter.StandardValuesCollection(((Project)context.Instance).Configurations);
+			public ICompilerResult Build (IProgressMonitor monitor, Project project)
+			{
+				project.DoPostBuild (monitor);
+				return null;
+			}
+			
+			public bool NeedsBuilding (Project project)
+			{
+				return false;
+			}
 		}
 	}
 }
