@@ -38,16 +38,28 @@ using System.Runtime.Serialization.Formatters.Binary;
 using System.Runtime.Remoting.Lifetime;
 using System.Reflection;
 using System.Collections;
+using Mono.Remoting.Channels.Unix;
 
 public class MonoDevelopProcessHost
 {
 	public static int Main (string[] args)
 	{
 		try {
-			Hashtable props = new Hashtable ();
-			props ["port"] = 0;
-			props ["name"] = "__internal_tcp";
-			ChannelServices.RegisterChannel (new TcpChannel (props, null, null));
+			string channel = Console.In.ReadLine ();
+			
+			string unixPath = null;
+			if (channel == "unix") {
+				unixPath = System.IO.Path.GetTempFileName ();
+				Hashtable props = new Hashtable ();
+				props ["path"] = unixPath;
+				props ["name"] = "__internal_unix";
+				ChannelServices.RegisterChannel (new UnixChannel (props, null, null));
+			} else {
+				Hashtable props = new Hashtable ();
+				props ["port"] = 0;
+				props ["name"] = "__internal_tcp";
+				ChannelServices.RegisterChannel (new TcpChannel (props, null, null));
+			}
 			
 			string sref = Console.In.ReadLine ();
 			byte[] data = Convert.FromBase64String (sref);
@@ -63,6 +75,9 @@ public class MonoDevelopProcessHost
 			}
 			rp.Dispose ();
 			
+			if (unixPath != null)
+				File.Delete (unixPath);
+			
 		} catch (Exception ex) {
 		}
 		
@@ -84,6 +99,7 @@ public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
 	
 	public RemoteProcessObject CreateInstance (Type type)
 	{
+		InitializeRuntime (type);
 		RemoteProcessObject proc = (RemoteProcessObject) Activator.CreateInstance (type);
 		proc.Attach (controller);
 		return proc;
@@ -94,12 +110,23 @@ public class ProcessHost: MarshalByRefObject, IProcessHost, ISponsor
 		try {
 			Type t = Type.GetType (fullTypeName);
 			if (t == null) throw new InvalidOperationException ("Type not found: " + fullTypeName);
+			InitializeRuntime (t);
 			return CreateInstance (t);
 		} catch (Exception ex) {
 			throw new InvalidOperationException ("Type not found: " + fullTypeName);
 		}
 	}
 	
+	void InitializeRuntime (Type type)
+	{
+		object[] ats = type.GetCustomAttributes (typeof(AddinDependencyAttribute), true);
+		if (ats.Length > 0) {
+			Runtime.Initialize ();
+			foreach (AddinDependencyAttribute at in ats)
+				Runtime.AddInService.PreloadAddin (null, at.Addin);
+		}
+	}
+	
 	public RemoteProcessObject CreateInstance (string assemblyPath, string typeName)
 	{
 		Assembly asm = Assembly.LoadFrom (assemblyPath);
