@@ -156,27 +156,56 @@ namespace CSharpBinding.Parser
 			Console.WriteLine("Here: Type is " + type.FullyQualifiedName);
 			return type;
 		}
-
-		public string MonodocResolver (string expression, int caretLineNumber, int caretColumn, string fileName, string fileContent) 
+		
+		public IClass ResolveExpressionType (ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit, Expression expr, int line, int col)
 		{
-			if (expression == null) {
-				return null;
-			}
-			expression = expression.TrimStart (null);
-			if (expression == "") {
-				return null;
-			}
-			IReturnType retType = internalResolve (expression, caretLineNumber, caretColumn, fileName, fileContent);
-			IClass retClass = parserContext.SearchType (retType.FullyQualifiedName, null, cu);
-			if (retClass == null) {
-				Console.WriteLine ("Retclass was null");
-				return null;
-			}
+			CSharpVisitor cSharpVisitor = new CSharpVisitor();
+			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
 			
-			Console.WriteLine (retClass.FullyQualifiedName);
-			return "T:" + retClass.FullyQualifiedName;
+			this.caretLine = line;
+			this.caretColumn = col;
+			
+			callingClass = GetInnermostClass();
+			
+			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor.Visit (fileCompilationUnit, null);
+			TypeVisitor typeVisitor = new TypeVisitor (this);
+			IReturnType type = expr.AcceptVisitor (typeVisitor, null) as IReturnType;
+			if (type != null)
+				return SearchType (type.FullyQualifiedName, cu);
+			else
+				return null;
+		}
+
+		public ILanguageItem ResolveIdentifier (IParserContext parserContext, string id, int line, int col, string fileName, string fileContent)
+		{
+			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
+			ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+			return ResolveIdentifier (fileCompilationUnit, id, line, col);
 		}
 		
+		public ILanguageItem ResolveIdentifier (ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit, string id, int line, int col)
+		{
+			ICSharpCode.SharpRefactory.Parser.Parser p = new ICSharpCode.SharpRefactory.Parser.Parser();
+			Lexer l = new Lexer(new StringReader(id));
+			Expression expr = p.ParseExpression(l);
+			
+			CSharpVisitor cSharpVisitor = new CSharpVisitor();
+			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			
+			this.caretLine = line;
+			this.caretColumn = col;
+			
+			callingClass = GetInnermostClass();
+			
+			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor.Visit (fileCompilationUnit, null);
+			
+			LanguageItemVisitor itemVisitor = new LanguageItemVisitor (this);
+			ILanguageItem item = expr.AcceptVisitor (itemVisitor, null) as ILanguageItem;
+			return item;
+		}
+
 		public ResolveResult Resolve (string expression, int caretLineNumber, int caretColumn, string fileName, string fileContent) 
 		{
 			if (expression == null) {
@@ -233,8 +262,8 @@ namespace CSharpBinding.Parser
 				if (n == null) {
 					return null;
 				}
-				ArrayList content = parserContext.GetNamespaceContents (n,true);
-				ArrayList classes = new ArrayList();
+				LanguageItemCollection content = parserContext.GetNamespaceContents (n,true);
+				LanguageItemCollection classes = new LanguageItemCollection();
 				for (int i = 0; i < content.Count; ++i) {
 					if (content[i] is IClass) {
 						classes.Add((IClass)content[i]);
@@ -246,10 +275,10 @@ namespace CSharpBinding.Parser
 			//Console.WriteLine("Returning Result!");
 			if (returnClass.FullyQualifiedName == "System.Void")
 				return null;
-			return new ResolveResult(returnClass, ListMembers(new ArrayList(), returnClass));
+			return new ResolveResult(returnClass, ListMembers(new LanguageItemCollection(), returnClass));
 		}
 		
-		ArrayList ListMembers(ArrayList members, IClass curType)
+		LanguageItemCollection ListMembers (LanguageItemCollection members, IClass curType)
 		{
 //			Console.WriteLine("LIST MEMBERS!!!");
 //			Console.WriteLine("showStatic = " + showStatic);
@@ -280,6 +309,7 @@ namespace CSharpBinding.Parser
 //					Console.WriteLine("Member added");
 				}
 			}
+			
 			foreach (IEvent e in curType.Events) {
 				if (MustBeShowen(curType, e)) {
 					members.Add(e);
@@ -355,6 +385,9 @@ namespace CSharpBinding.Parser
 //				Console.WriteLine("IsAccessible");
 				return true;
 			}
+			if (callingClass == null)
+				return false;
+
 			return c.FullyQualifiedName == callingClass.FullyQualifiedName;
 		}
 		
@@ -429,74 +462,114 @@ namespace CSharpBinding.Parser
 		}
 		
 		// no methods or indexer
-		public IReturnType SearchMember(IReturnType type, string memberName)
+		public IReturnType SearchMember (IReturnType type, string memberName)
 		{
-			if (type == null || memberName == null || memberName == "") {
+			IClass curType;
+			IDecoration member;
+			
+			if (!SearchClassMember (type, memberName, false, out curType, out member))
 				return null;
+			
+			if (member is IField) {
+				if (curType.ClassType == ClassType.Enum)
+					return type; // enum members have the type of the enum
+				else
+					return ((IField)member).ReturnType;
 			}
-//			Console.WriteLine("searching member {0} in {1}", memberName, type.Name);
-			IClass curType = SearchType(type.FullyQualifiedName, cu);
-			if (curType == null) {
-//				Console.WriteLine("Type not found in SearchMember");
+			else if (member is IClass) {
+				return new ReturnType (((IClass)member).FullyQualifiedName);
+			}
+			else if (member is IProperty) {
+				return ((IProperty)member).ReturnType;
+			}
+			else if (member is IEvent) {
+				return ((IEvent)member).ReturnType;
+			}
+			
+			throw new InvalidOperationException ("Unknown member type:" + member);
+		}
+		
+		public IDecoration SearchClassMember (IReturnType type, string memberName, bool includeMethods)
+		{
+			IDecoration member;
+			IClass curType;
+			if (SearchClassMember (type, memberName, includeMethods, out curType, out member))
+				return member;
+			else
 				return null;
-			}
-			if (type.PointerNestingLevel != 0) {
-				return null;
-			}
-			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0) {
+		}
+		
+		bool SearchClassMember (IReturnType type, string memberName, bool includeMethods, out IClass curType, out IDecoration member)
+		{
+			curType = null;
+			member = null;
+			
+			if (type == null || memberName == null || memberName == "")
+				return false;
+			
+			curType = SearchType (type.FullyQualifiedName, cu);
+			if (curType == null)
+				return false;
+
+			if (type.PointerNestingLevel != 0)
+				return false;
+
+			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0)
 				curType = SearchType("System.Array", null);
-			}
+
 			if (curType.ClassType == ClassType.Enum) {
 				foreach (IField f in curType.Fields) {
 					if (f.Name == memberName && MustBeShowen(curType, f)) {
 						showStatic = false;
-						return type; // enum members have the type of the enum
+						member = f; // enum members have the type of the enum
+						return true;
 					}
 				}
 			}
 			if (showStatic) {
-//				Console.WriteLine("showStatic == true");
 				foreach (IClass c in curType.InnerClasses) {
 					if (c.Name == memberName && IsAccessible(curType, c)) {
-						return new ReturnType(c.FullyQualifiedName);
+						member = c;
+						return true;
 					}
 				}
 			}
-//			Console.WriteLine("#Properties " + curType.Properties.Count);
 			foreach (IProperty p in curType.Properties) {
-//				Console.WriteLine("checke Property " + p.Name);
-//				Console.WriteLine("member name " + memberName);
 				if (p.Name == memberName && MustBeShowen(curType, p)) {
-//					Console.WriteLine("Property found " + p.Name);
 					showStatic = false;
-					return p.ReturnType;
+					member = p;
+					return true;
 				}
 			}
 			foreach (IField f in curType.Fields) {
-//				Console.WriteLine("checke Feld " + f.Name);
-//				Console.WriteLine("member name " + memberName);
 				if (f.Name == memberName && MustBeShowen(curType, f)) {
-//					Console.WriteLine("Field found " + f.Name);
 					showStatic = false;
-					return f.ReturnType;
+					member = f;
+					return true;
 				}
 			}
 			foreach (IEvent e in curType.Events) {
 				if (e.Name == memberName && MustBeShowen(curType, e)) {
 					showStatic = false;
-					return e.ReturnType;
+					member = e;
+					return true;
+				}
+			}
+			if (includeMethods) {
+				foreach (IMethod m in curType.Methods) {
+					if (m.Name == memberName && MustBeShowen(curType, m)) {
+						showStatic = false;
+						member = m;
+						return true;
+					}
 				}
 			}
 			foreach (string baseType in curType.BaseTypes) {
 				IClass c = parserContext.GetClass (baseType, true, true);
-				if (c != null) {
-					IReturnType erg = SearchMember(new ReturnType(c.FullyQualifiedName), memberName);
-					if (erg != null) {
-						return erg;
-					}
-				}
+				if (c != null)
+					return SearchClassMember (new ReturnType(c.FullyQualifiedName), memberName, includeMethods, out curType, out member);
 			}
-			return null;
+			return false;
 		}
 		
 		bool IsInside(Point between, Point start, Point end)
@@ -529,10 +602,10 @@ namespace CSharpBinding.Parser
 //			
 //			Console.WriteLine("LookUpTable has {0} entries", lookupTableVisitor.variables.Count);
 //			Console.WriteLine("Listing Variables:");
-			IDictionaryEnumerator enumerator = lookupTableVisitor.variables.GetEnumerator();
-			while (enumerator.MoveNext()) {
-				Console.WriteLine(enumerator.Key);
-			}
+//			IDictionaryEnumerator enumerator = lookupTableVisitor.variables.GetEnumerator();
+//			while (enumerator.MoveNext()) {
+//				Console.WriteLine(enumerator.Key);
+//			}
 //			Console.WriteLine("end listing");
 			ArrayList variables = (ArrayList)lookupTableVisitor.variables[name];
 			if (variables == null || variables.Count <= 0) {
@@ -556,6 +629,67 @@ namespace CSharpBinding.Parser
 			return found;
 		}
 		
+		/// <remarks>
+		/// does the dynamic lookup for the id
+		/// </remarks>
+		public ILanguageItem IdentifierLookup (string id)
+		{
+			// try if it exists a variable named id
+			ReturnType variable = SearchVariable (id);
+			if (variable != null) {
+				return new LocalVariable (id, variable, "");
+			}
+			
+			if (callingClass == null) {
+				return null;
+			}
+			
+			//// somehow search in callingClass fields is not returning anything, so I am searching here once again
+			foreach (IField f in callingClass.Fields) {
+				if (f.Name == id) {
+					return f;
+				}
+			}
+		
+			// try if typeName is a method parameter
+			IParameter p = SearchMethodParameter (id);
+			if (p != null) {
+				return p;
+			}
+			
+			// check if typeName == value in set method of a property
+			if (id == "value") {
+				IProperty pr = SearchProperty();
+				if (pr != null) {
+					return pr;
+				}
+			}
+			
+			// try if there exists a nonstatic member named typeName
+			showStatic = false;
+			IClass cls;
+			IDecoration member;
+			if (SearchClassMember (callingClass == null ? null : new ReturnType(callingClass.FullyQualifiedName), id, true, out cls, out member)) {
+				return member;
+			}
+			
+			// try if there exists a static member named typeName
+			showStatic = true;
+			if (SearchClassMember (callingClass == null ? null : new ReturnType(callingClass.FullyQualifiedName), id, true, out cls, out member)) {
+				showStatic = false;
+				return member;
+			}
+			
+			// try if there exists a static member in outer classes named typeName
+			foreach (IClass c in GetOuterClasses()) {
+				if (SearchClassMember (callingClass == null ? null : new ReturnType(c.FullyQualifiedName), id, true, out cls, out member)) {
+					showStatic = false;
+					return member;
+				}
+			}
+			return null;
+		}
+		
 		/// <remarks>
 		/// does the dynamic lookup for the typeName
 		/// </remarks>
@@ -585,20 +719,20 @@ namespace CSharpBinding.Parser
 			//// end of mod for search in Fields
 		
 			// try if typeName is a method parameter
-			IReturnType p = SearchMethodParameter(typeName);
+			IParameter p = SearchMethodParameter(typeName);
 			if (p != null) {
 //				Console.WriteLine("MethodParameter Found");
 				showStatic = false;
-				return p;
+				return p.ReturnType;
 			}
 //			Console.WriteLine("No Parameter found");
 			
 			// check if typeName == value in set method of a property
 			if (typeName == "value") {
-				p = SearchProperty();
-				if (p != null) {
+				IProperty pr = SearchProperty();
+				if (pr != null) {
 					showStatic = false;
-					return p;
+					return pr.ReturnType;
 				}
 			}
 //			Console.WriteLine("No Property found");
@@ -653,29 +787,27 @@ namespace CSharpBinding.Parser
 			return null;
 		}
 		
-		IReturnType SearchProperty()
+		IProperty SearchProperty ()
 		{
 			IProperty property = GetProperty();
 			if (property == null) {
 				return null;
 			}
 			if (property.SetterRegion != null && property.SetterRegion.IsInside(caretLine, caretColumn)) {
-				return property.ReturnType;
+				return property;
 			}
 			return null;
 		}
 		
-		IReturnType SearchMethodParameter(string parameter)
+		IParameter SearchMethodParameter(string parameter)
 		{
 			IMethod method = GetMethod();
 			if (method == null) {
-				Console.WriteLine("Method not found");
 				return null;
 			}
 			foreach (IParameter p in method.Parameters) {
 				if (p.Name == parameter) {
-					Console.WriteLine("Parameter found");
-					return p.ReturnType;
+					return p;
 				}
 			}
 			return null;
@@ -782,7 +914,9 @@ namespace CSharpBinding.Parser
 		{
 			if (cu != null) {
 				foreach (IClass c in cu.Classes) {
-					if (c != null && c.Region != null && c.Region.IsInside(caretLine, caretColumn)) {
+					if (c != null && ((c.Region != null && c.Region.IsInside(caretLine, caretColumn)) ||
+						              (c.BodyRegion != null && c.BodyRegion.IsInside(caretLine, caretColumn))))
+					{
 						return GetInnermostClass(c);
 					}
 				}
@@ -799,7 +933,7 @@ namespace CSharpBinding.Parser
 				return GetResolvedClass (curClass);
 			}
 			foreach (IClass c in curClass.InnerClasses) {
-				if (c != null && c.Region != null && c.Region.IsInside(caretLine, caretColumn)) {
+				if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 					return GetInnermostClass(c);
 				}
 			}
@@ -817,7 +951,7 @@ namespace CSharpBinding.Parser
 			ClassCollection classes = new ClassCollection();
 			if (cu != null) {
 				foreach (IClass c in cu.Classes) {
-					if (c != null && c.Region != null && c.Region.IsInside(caretLine, caretColumn)) {
+					if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 						if (c != GetInnermostClass()) {
 							GetOuterClasses(classes, c);
 							classes.Add(GetResolvedClass (c));
@@ -834,7 +968,7 @@ namespace CSharpBinding.Parser
 		{
 			if (curClass != null) {
 				foreach (IClass c in curClass.InnerClasses) {
-					if (c != null && c.Region != null && c.Region.IsInside(caretLine, caretColumn)) {
+					if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 						if (c != GetInnermostClass()) {
 							GetOuterClasses(classes, c);
 							classes.Add(GetResolvedClass (c));
@@ -851,9 +985,9 @@ namespace CSharpBinding.Parser
 			return parserContext.GetClass (cls.FullyQualifiedName);
 		}
 
-		public ArrayList IsAsResolve (string expression, int caretLine, int caretColumn, string fileName, string fileContent)
+		public LanguageItemCollection IsAsResolve (string expression, int caretLine, int caretColumn, string fileName, string fileContent)
 		{
-			ArrayList result = new ArrayList ();
+			LanguageItemCollection result = new LanguageItemCollection ();
 			this.caretLine = caretLine;
 			this.caretColumn = caretColumn;
 			
@@ -906,9 +1040,12 @@ namespace CSharpBinding.Parser
 			return result;
 		}
 		
-		public ArrayList CtrlSpace (int caretLine, int caretColumn, string fileName)
+		public LanguageItemCollection CtrlSpace (int caretLine, int caretColumn, string fileName)
 		{
-			ArrayList result = new ArrayList(TypeReference.PrimitiveTypes);
+			LanguageItemCollection result = new LanguageItemCollection ();
+			foreach (string pt in TypeReference.PrimitiveTypes)
+				result.Add (new Namespace (pt));
+
 			this.caretLine = caretLine;
 			this.caretColumn = caretColumn;
 			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
@@ -930,7 +1067,7 @@ namespace CSharpBinding.Parser
 				if (variables != null && variables.Count > 0) {
 					foreach (LocalLookupVariable v in variables) {
 						if (IsInside(new Point(caretColumn, caretLine), v.StartPos, v.EndPos)) {
-							result.Add(new Parameter (name, new ReturnType (v.TypeRef.SystemType)));
+							result.Add(new Parameter (null, name, new ReturnType (v.TypeRef.SystemType)));
 							break;
 						}
 					}
@@ -947,7 +1084,7 @@ namespace CSharpBinding.Parser
 						result.AddRange(parserContext.GetNamespaceContents (name, true));
 					}
 					foreach (string alias in u.Aliases.Keys) {
-						result.Add(alias);
+						result.Add(new Namespace (alias));
 					}
 				}
 			}
