@@ -16,6 +16,7 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Search;
 using MonoDevelop.Ide.Codons;
 using MonoDevelop.SourceEditor.Document;
+using MonoDevelop.Projects.Text;
 
 using Gtk;
 using GtkSourceView;
@@ -51,11 +52,6 @@ namespace MonoDevelop.SourceEditor.Gui
 			return false;
 		}
 		
-		public virtual bool CanCreateContentForLanguage (string language)
-		{
-			return true;
-		}
-		
 		public virtual IViewContent CreateContentForFile (string fileName)
 		{
 			SourceEditorDisplayBindingWrapper w = new SourceEditorDisplayBindingWrapper ();
@@ -64,40 +60,20 @@ namespace MonoDevelop.SourceEditor.Gui
 			return w;
 		}
 		
-		public virtual IViewContent CreateContentForLanguage (string language, string content)
-		{
-			return CreateContentForLanguage (language, content, null);
-		}
-		
-		public virtual IViewContent CreateContentForLanguage (string language, string content, string new_file_name)
+		public virtual IViewContent CreateContentForMimeType (string mimeType, Stream content)
 		{
+			StreamReader sr = new StreamReader (content);
+			string text = sr.ReadToEnd ();
+			sr.Close ();
+			
 			SourceEditorDisplayBindingWrapper w = new SourceEditorDisplayBindingWrapper ();
-			
-			switch (language.ToUpper ()) {
-				case "C#":
-					language = "text/x-csharp";
-					break;
-				case "JAVA":
-					language = "text/x-java";
-					break;
-				//case language "VBNET":
-				//	language = "text/x-vbnet";
-				//	break;
-				case "NEMERLE":
-					language = "text/x-nemerle";
-					break;
-				default:
-					language = "text/plain";
-					break;
-			}
-			
-			w.LoadString (language, sps.Parse (content));
+			w.LoadString (mimeType, sps.Parse (text));
 			return w;
 		}	
 	}
 	
 	public class SourceEditorDisplayBindingWrapper : AbstractViewContent,
-		IEditable, IPositionable, IBookmarkBuffer, IDebuggableEditor, ICodeStyleOperations,
+		IEditableTextBuffer, IPositionable, IBookmarkBuffer, IDebuggableEditor, ICodeStyleOperations,
 		IDocumentInformation
 	{
 		VBox mainBox;
@@ -257,7 +233,16 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		void OnModifiedChanged (object o, EventArgs e)
 		{
-			this.IsDirty = se.Buffer.Modified;
+			base.IsDirty = se.Buffer.Modified;
+		}
+		
+		public override bool IsDirty {
+			get {
+				return base.IsDirty;
+			}
+			set {
+				se.Buffer.Modified = value;
+			}
 		}
 		
 		public override bool IsReadOnly
@@ -271,7 +256,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		{
 			if (warnOverwrite) {
 				if (fileName == ContentName) {
-					if (!Services.MessageService.AskQuestion (string.Format (GettextCatalog.GetString ("The file {0} has been changed outside of MonoDevelop. Are you sure you want to overwrite the file?"), fileName),"MonoDevelop"))
+					if (!Services.MessageService.AskQuestion (string.Format (GettextCatalog.GetString ("This file {0} has been changed outside of MonoDevelop. Are you sure you want to overwrite the file?"), fileName),"MonoDevelop"))
 						return;
 				}
 				warnOverwrite = false;
@@ -357,11 +342,11 @@ namespace MonoDevelop.SourceEditor.Gui
 				HBox box = new HBox ();
 				reloadBar.PackStart (box, true, true, 10);
 				
-				Button b1 = new Button ("Reload");
+				Button b1 = new Button (GettextCatalog.GetString("Reload"));
 				box.PackStart (b1, false, false, 5);
 				b1.Clicked += new EventHandler (ClickedReload);
 				
-				Button b2 = new Button ("Ignore");
+				Button b2 = new Button (GettextCatalog.GetString("Ignore"));
 				box.PackStart (b2, false, false, 5);
 				b2.Clicked += new EventHandler (ClickedIgnore);
 
@@ -414,7 +399,6 @@ namespace MonoDevelop.SourceEditor.Gui
 					IFormattingStrategy[] formatter = (IFormattingStrategy[])(AddInTreeSingleton.AddInTree.GetTreeNode(path).BuildChildItems(this)).ToArray(typeof(IFormattingStrategy));
 					if (formatter != null && formatter.Length > 0) {
 						se.View.fmtr = formatter[0];
-						Console.WriteLine ("set formatter to {0}", formatter[0]);
 						return;
 					}
 				}
@@ -439,11 +423,15 @@ namespace MonoDevelop.SourceEditor.Gui
 				se.Buffer.LoadText (val);
 		}
 		
-#region IEditable
+#region IEditableTextBuffer
 		public IClipboardHandler ClipboardHandler {
 			get { return se.Buffer; }
 		}
 		
+		public string Name {
+			get { return ContentName; }
+		}
+		
 		string cachedText;
 		GLib.IdleHandler bouncingDelegate;
 		
@@ -505,23 +493,21 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 		}
 		
-		public object GetPositionFromLineColumn (int line, int column)
+		public int GetPositionFromLineColumn (int line, int column)
 		{
 			TextIter itr = se.Buffer.GetIterAtLine (line - 1);
 			itr.LineOffset = column - 1 > itr.CharsInLine ? itr.CharsInLine - 1 : column - 1;
-			return itr;
+			return itr.Offset;
 		}
 		
-		public void InsertText (object position, string text)
+		public void InsertText (int position, string text)
 		{
-			TextIter it = (TextIter) position;
-			se.Buffer.Insert (it.Offset, text);
+			se.Buffer.Insert (position, text);
 		}
 		
-		public void DeleteText (object position, int length)
+		public void DeleteText (int pos, int length)
 		{
-			int pos = ((TextIter)position).Offset;
-			se.Buffer.Delete (pos, pos + length);
+			se.Buffer.Delete (pos, length);
 		}
 		
 		public event EventHandler TextChanged {
@@ -613,65 +599,66 @@ namespace MonoDevelop.SourceEditor.Gui
 		}
 #endregion 
 
-		public object CursorPosition {
-			get { return se.Buffer.GetIterAtMark (se.Buffer.InsertMark); }
-			set { se.Buffer.MoveMark (se.Buffer.InsertMark, (TextIter) value); }
+		public int CursorPosition {
+			get { return se.Buffer.GetIterAtMark (se.Buffer.InsertMark).Offset; }
+			set { se.Buffer.MoveMark (se.Buffer.InsertMark, se.Buffer.GetIterAtOffset (value)); }
 		}
 
-		public object GetPositionFromOffset (int offset)
+		public void Select (int startPosition, int endPosition)
 		{
-			return se.Buffer.GetIterAtOffset (offset);
+			se.Buffer.MoveMark (se.Buffer.InsertMark, se.Buffer.GetIterAtOffset (startPosition));
+			se.Buffer.MoveMark (se.Buffer.SelectionBound, se.Buffer.GetIterAtOffset (endPosition));
 		}
 		
-		public int GetOffsetFromPosition (object position)
-		{
-			return ((TextIter)position).Offset;
-		}
-		
-		public void Select (object startPosition, object endPosition)
-		{
-			se.Buffer.MoveMark (se.Buffer.InsertMark, (TextIter) startPosition);
-			se.Buffer.MoveMark (se.Buffer.SelectionBound, (TextIter) endPosition);
-		}
-		
-		public object SelectionStartPosition {
+		public int SelectionStartPosition {
 			get {
 				TextIter p1 = se.Buffer.GetIterAtMark (se.Buffer.InsertMark);
 				TextIter p2 = se.Buffer.GetIterAtMark (se.Buffer.SelectionBound);
-				if (p1.Offset < p2.Offset) return p1;
-				else return p2;
+				if (p1.Offset < p2.Offset) return p1.Offset;
+				else return p2.Offset;
 			}
 		}
 		
-		public object SelectionEndPosition {
+		public int SelectionEndPosition {
 			get {
 				TextIter p1 = se.Buffer.GetIterAtMark (se.Buffer.InsertMark);
 				TextIter p2 = se.Buffer.GetIterAtMark (se.Buffer.SelectionBound);
-				if (p1.Offset > p2.Offset) return p1;
-				else return p2;
+				if (p1.Offset > p2.Offset) return p1.Offset;
+				else return p2.Offset;
 			}
 		}
 		
-		public void ShowPosition (object position)
+		public void GetLineColumnFromPosition (int position, out int line, out int column)
 		{
-			se.View.ScrollToIter ((TextIter)position, 0.3, false, 0, 0);
+			TextIter it = se.Buffer.GetIterAtOffset (position);
+			line = it.Line + 1;
+			column = it.LineOffset;
 		}
 		
-		public string GetText (object startPosition, object endPosition)
+		public void ShowPosition (int position)
 		{
-			return se.Buffer.GetText ((TextIter)startPosition, (TextIter)endPosition, true);
+			se.View.ScrollToIter (se.Buffer.GetIterAtOffset (position), 0.3, false, 0, 0);
+		}
+		
+		public string GetText (int startPosition, int endPosition)
+		{
+			return se.Buffer.GetText (se.Buffer.GetIterAtOffset (startPosition), se.Buffer.GetIterAtOffset (endPosition), true);
+		}
+		
+		int ITextFile.Length {
+			get { return se.Buffer.Length; }
 		}
 
-		public void SetBookmarked (object position, bool mark)
+		public void SetBookmarked (int position, bool mark)
 		{
-			int line = ((TextIter)position).Line;
+			int line = se.Buffer.GetIterAtOffset (position).Line;
 			if (se.Buffer.IsBookmarked (line) != mark)
 				se.Buffer.ToggleBookmark (line);
 		}
 		
-		public bool IsBookmarked (object position)
+		public bool IsBookmarked (int position)
 		{
-			int line = ((TextIter)position).Line;
+			int line = se.Buffer.GetIterAtOffset (position).Line;
 			return se.Buffer.IsBookmarked (line);
 		}
 		
@@ -795,10 +782,14 @@ namespace MonoDevelop.SourceEditor.Gui
 	class SourceViewTextIterator: ForwardTextIterator
 	{
 		bool initialBackwardsPosition;
+		bool hasWrapped;
 		
 		public SourceViewTextIterator (IDocumentInformation docInfo, Gtk.TextView document, int endOffset)
 		: base (docInfo, document, endOffset)
 		{
+			// Make sure the iterator is ready for use
+			this.MoveAhead(1);
+			this.hasWrapped = false;
 		}
 		
 		public override bool SupportsSearch (SearchOptions options, bool reverse)
@@ -822,53 +813,46 @@ namespace MonoDevelop.SourceEditor.Gui
 				Position++;
 				initialBackwardsPosition = false;
 			}
-			
-			TextIter it = Buffer.GetIterAtOffset (DocumentOffset);
-			
-			int limitOffset = EndOffset;
-			if (reverse)
-				limitOffset = (limitOffset + text.Length - 1) % BufferLength;
-			else
-				limitOffset = (limitOffset + 1) % BufferLength;
-
+							
 			// Use special search flags that work for both the old and new API
 			// of gtksourceview (the enum values where changed in the API).
 			// See bug #75770
 			SourceSearchFlags flags = options.IgnoreCase ? (SourceSearchFlags)7 : (SourceSearchFlags)1;
 			
-			Gtk.TextIter matchStart, matchEnd;
-			bool res;
-			
-			Gtk.TextIter limit;
+			Gtk.TextIter matchStart, matchEnd, limit;
+								
 			
 			if (reverse) {
-				if (DocumentOffset <= EndOffset)
+				if (!hasWrapped)
 					limit = Buffer.StartIter;
 				else
-					limit = Buffer.GetIterAtOffset (limitOffset);
+					limit = Buffer.GetIterAtOffset (EndOffset);
 			} else {
-				if (DocumentOffset >= EndOffset)
+				if (!hasWrapped)
 					limit = Buffer.EndIter;
 				else
-					limit = Buffer.GetIterAtOffset (limitOffset);
+					limit = Buffer.GetIterAtOffset (EndOffset + text.Length);
 			}
 			
 			// machEnd is the position of the last matched char + 1
 			// When searching forward, the limit check is: matchEnd < limit
 			// When searching backwards, the limit check is: matchEnd > limit
 			
-			res = Find (reverse, it, text, flags, out matchStart, out matchEnd, limit);
+			TextIter iterator = Buffer.GetIterAtOffset (DocumentOffset);
+			bool res = Find (reverse, iterator, text, flags, out matchStart, out matchEnd, limit);
 			
-			if (!res) {
-				// Not found in the first half of the document, try not the other half
-				if (reverse && DocumentOffset <= EndOffset) {
-					it = Buffer.EndIter;
-					limit = Buffer.GetIterAtOffset (limitOffset);
-					res = Find (true, it, text, flags, out matchStart, out matchEnd, limit);
-				} else if (!reverse && DocumentOffset >= EndOffset) {
-					it = Buffer.StartIter;
-					limit = Buffer.GetIterAtOffset (limitOffset);
-					res = Find (false, it, text, flags, out matchStart, out matchEnd, limit);
+			if (!res && !hasWrapped) {
+				
+				hasWrapped = true;																
+								
+				// Not found in the first half of the document, try the other half
+				if (reverse && DocumentOffset <= EndOffset) {					
+					limit = Buffer.GetIterAtOffset (EndOffset);
+					res = Find (true, Buffer.EndIter, text, flags, out matchStart, out matchEnd, limit);
+				// Not found in the second half of the document, try the other half
+				} else if (!reverse && DocumentOffset >= EndOffset) {										
+					limit = Buffer.GetIterAtOffset (EndOffset + text.Length);									
+					res = Find (false, Buffer.StartIter, text, flags, out matchStart, out matchEnd, limit);
 				}
 			}
 			
