@@ -136,10 +136,25 @@ namespace MonoDevelop.SourceEditor.Gui
 					
 					menu.Popup (null, null, null, 3, e.Time);
 				}
+			} else if (e.Button == 3 && buf.GetSelectedText ().Length == 0) {
+				int x, y;
+				WindowToBufferCoords (Gtk.TextWindowType.Text, (int) e.X, (int) e.Y, out x, out y);
+				buf.PlaceCursor (GetIterAtLocation (x, y));		
 			}
+			
 			return base.OnButtonPressEvent (e);
 		}
 		
+		protected override void OnPopulatePopup (Menu menu)
+		{
+			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet ("/SharpDevelop/ViewContent/DefaultTextEditor/ContextMenu");
+			if (cset.Count > 0) {
+				cset.AddItem (Command.Separator);
+				IdeApp.CommandService.InsertOptions (menu, cset, 0);
+			}
+			base.OnPopulatePopup (menu);
+		}
+		
 		public void ShowBreakpointAt (int linenumber)
 		{
 			if (!buf.IsMarked (linenumber, SourceMarkerType.BreakpointMark))
@@ -257,6 +272,7 @@ namespace MonoDevelop.SourceEditor.Gui
 					case '\r':
 					case '\n':
 					case '.':
+					case ';':
 					case '(':
 					case '[':
 						triggerIter = insertIter;
@@ -273,14 +289,16 @@ namespace MonoDevelop.SourceEditor.Gui
 			insertIter = triggerIter;
 			string fileName = ParentEditor.DisplayBinding.ContentName;
 			IExpressionFinder expressionFinder = GetParserContext ().GetExpressionFinder(fileName);
-			string expression    = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset(this, insertIter.Offset) : expressionFinder.FindExpression(buf.GetText(buf.StartIter, insertIter, true), insertIter.Offset - 2);
+			string expression    = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset(this, insertIter.Offset) : expressionFinder.FindExpression(buf.GetText(buf.StartIter, insertIter, true), insertIter.Offset - 2).Expression;
 			if (expression == null) return false;
 			Console.WriteLine ("Expression: {" + expression + "}");
-			string type = GetParserContext ().MonodocResolver (expression, insertIter.Line + 1, insertIter.LineOffset + 1, fileName, buf.Text);
-			if (type == null || type.Length == 0)
+
+			ILanguageItem languageItem = GetParserContext ().ResolveIdentifier(expression, insertIter.Line + 1, insertIter.LineOffset + 1, fileName, buf.Text);
+			
+			if (languageItem == null)
 				return false;
 
-			IdeApp.HelpOperations.ShowHelp (type);
+			IdeApp.HelpOperations.ShowHelp (MonoDevelop.Projects.Services.DocumentationService.GetHelpUrl(languageItem));
 			
 			return true;
 		}
@@ -486,7 +504,7 @@ namespace MonoDevelop.SourceEditor.Gui
 			TextIter iter = buf.GetIterAtMark (buf.InsertMark);
 
 			// preserve offset in line
-			int offset = iter.LineOffset;
+			int offset = 0;
 			int chars = fmtr.IndentLine (this, iter.Line);
 			offset += chars;
 
